@@ -438,6 +438,13 @@ do_cancel (CANON_Scanner * s)
   return (SANE_STATUS_CANCELLED);
 }
 
+static SANE_Status
+attach_one (const char *dev)
+{
+  attach (dev, 0);
+  return SANE_STATUS_GOOD;
+}
+
 SANE_Status
 sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 {
@@ -475,7 +482,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 	}
       fclose (fp);
     }
-  attach (devnam, NULL);
+  sanei_config_attach_matching_devices (devnam, attach_one);
   DBG (11, "<< sane_init\n");
   return SANE_STATUS_GOOD;
 }
