@@ -1,6 +1,6 @@
 /* ------------------------------------------------------------------------- */
 /* sane - Scanner Access Now Easy.
-   coolscan.c , version 1
+   coolscan.c , version .3
 
    This file is part of the SANE package.
 
@@ -68,7 +68,7 @@
 # include <lalloca.h>		/* MUST come first for AIX! */
 #endif
 
-#include "sane/config.h"	 /*get inline #define'd if necessary... */
+#include "sane/config.h"
 #include <lalloca.h>
 
 #include <errno.h>
@@ -115,7 +115,7 @@ static const SANE_Int resolution_list[] =
 #define	COOLSCAN_MAX_RETRY	25
 
 
-static int coolscan_get_internal_info (Coolscan_t *);
+static int get_internal_info (Coolscan_t *);
 static void coolscan_get_inquiry_values (Coolscan_t *);
 static void hexdump (int level, char *comment, unsigned char *p, int l);
 static int swap_res (Coolscan_t * s);
@@ -565,19 +565,19 @@ coolscan_set_window_param (Coolscan_t * s, int prescan)
       set_WD_shading (buffer_r, WD_Shading_ON);		/* default for non-manufacturing */
 
       if (1 == s->LS)
-	{			/* Analog gamma reserved on LS-1 */
+	{			/* Analog gamma reserved on LS-1000 */
 	  set_WD_analog_gamma_R (buffer_r, 0);
 	  set_WD_analog_gamma_G (buffer_r, 0);
 	  set_WD_analog_gamma_R (buffer_r, 0);
 	}
       else
 	{
-	  int foo;
-
 	  /* Quote spec: "It is recomended that analog gamma bits 5, 4 and 3 be
 	   * set to 1 (OFF) when the object type of byte 48 is positive and the
 	   * gamma specificateion of byte 51 is linear, and to 0 (ON) in all
 	   * other cases." */
+	  /*
+	  int foo;
 	  if ((buffer_r[48] == WD_Positive) && (buffer_r[51] == WD_Linear))
 	    foo = WD_Analog_Gamma_OFF;
 	  else
@@ -585,7 +585,10 @@ coolscan_set_window_param (Coolscan_t * s, int prescan)
 	  set_WD_analog_gamma_R (buffer_r, foo);
 	  set_WD_analog_gamma_G (buffer_r, foo);
 	  set_WD_analog_gamma_B (buffer_r, foo);
-
+	  */
+	  set_WD_analog_gamma_R (buffer_r, s->analog_gamma_r);
+	  set_WD_analog_gamma_G (buffer_r, s->analog_gamma_g);
+	  set_WD_analog_gamma_B (buffer_r, s->analog_gamma_b);
 	  if (s->gamma_bind)
 	    {
 	      set_WD_LUT_R (buffer_r, 1);
@@ -616,6 +619,7 @@ coolscan_set_window_param (Coolscan_t * s, int prescan)
       set_WD_shift_G (buffer_r, s->shift_G);
       set_WD_shift_B (buffer_r, s->shift_B);
 
+
       /* FIXME: LUT-[RGB] */
       /* FIXME: stop on/off */
     }
@@ -677,7 +681,7 @@ coolscan_get_window_param (Coolscan_t * s)
 		     s->buffer, translen);
 
   buf = s->buffer + window_parameter_data_block.size;
-  hexdump (15, "Window get", buf, 117);
+  hexdump (10, "Window get", buf, 117);
 
   /* BTC */
   s->brightness = get_WD_brightness (buf);	/* brightness */
@@ -715,7 +719,7 @@ coolscan_get_window_param (Coolscan_t * s)
 }
 
 static int
-coolscan_prescan (Coolscan_t * s)
+prescan (Coolscan_t * s)
 {
   int ret;
 
@@ -941,7 +945,7 @@ coolscan_check_values (Coolscan_t * s)
 }
 
 static int
-coolscan_get_internal_info (Coolscan_t * s)
+get_internal_info (Coolscan_t * s)
 {
   int ret;
 
@@ -998,11 +1002,12 @@ coolscan_get_internal_info (Coolscan_t * s)
        "\tadbits=%d\toutputbits=%d\tmaxres=%d\txmax=%d\tymax=%d\n"
        "\txmaxpix=%d\tymaxpix=%d\tycurrent=%d\tcurrentfocus=%d\n"
        "\tautofeeder=%s\tanaloggamma=%s\tcurrentscanpitch=%d\n"
-       "\twbetr_r=%d\twebtr_g=%d\twebtr_b=%d\n"
-       "\tpretv_r=%d\tpretv_r=%d\tpretv_g=%d\n"
-       "\tcetv_r=%d\tcetv_g=%d\tcetv_b=%d\n"
-       "\tietu_r=%d\tietu_g=%d\tietu_b=%d\n"
-       "\tlimitcondition=0x%x\t\toffsetdata_[rgb]=0x%x 0x%x 0x%x\n"
+       "\tWhite balance exposure time var [RGB]=\t%d %d %d\n"
+       "\tPrescan result exposure time var [RGB]=\t%d %d %d\n"
+       "\tCurrent exposure time var.[RGB]=\t%d %d %d\n"
+       "\tInternal exposure time unit[RGB]=\t%d %d %d\n"
+       "\toffsetdata_[rgb]=\t0x%x 0x%x 0x%x\n"
+       "\tlimitcondition=0x%x\n"
        "\tdevice error code = 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n"
        "\tpower-on errors = 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
        s->adbits, s->outputbits, s->maxres, s->xmax, s->ymax,
@@ -1012,8 +1017,8 @@ coolscan_get_internal_info (Coolscan_t * s)
        s->wbetr_r, s->webtr_g, s->webtr_b, s->pretv_r, s->pretv_g,
        s->pretv_r, s->cetv_r, s->cetv_g, s->cetv_b, s->ietu_r,
        s->ietu_g, s->ietu_b,
-       s->limitcondition, s->offsetdata_r, s->offsetdata_g,
-       s->offsetdata_b,
+       s->offsetdata_r, s->offsetdata_g, s->offsetdata_b,
+       s->limitcondition, 
        s->derr[0], s->derr[1], s->derr[2], s->derr[3], s->derr[4],
        s->derr[5], s->derr[6], s->derr[7],
        s->power_on_errors[0], s->power_on_errors[1],
@@ -1058,7 +1063,7 @@ coolscan_initialize_values (Coolscan_t * s)
 
   select_MUD (s);		/* must be before mode_sense */
   coolscan_mode_sense (s);	/* Obtain MUD (Measurement Unit Divisor) */
-  coolscan_get_internal_info (s);	/* MUST be called first. */
+  get_internal_info (s);	/* MUST be called first. */
 
   s->cont = 0;			/* do not continue if scanner is unknown */
   s->verbose = 2;		/* 1=verbose,2=very verbose */
@@ -1333,7 +1338,12 @@ attach_scanner (const char *devicename, Coolscan_t ** devp)
   if (NULL == (dev = malloc (sizeof (*dev))))
     return SANE_STATUS_NO_MEM;
 
-  if ((dev->buffer = malloc (sanei_scsi_max_request_size)) == NULL)
+	
+  dev->row_bufsize = (sanei_scsi_max_request_size < (64 * 1024)) ?
+  	sanei_scsi_max_request_size : 64 * 1024;
+ 
+  if ((dev->buffer = malloc (dev->row_bufsize)) == NULL)
+/*  if ((dev->buffer = malloc (sanei_scsi_max_request_size)) == NULL)*/
     return SANE_STATUS_NO_MEM;
 
   dev->devicename = strdup (devicename);
@@ -1391,6 +1401,12 @@ attach_scanner (const char *devicename, Coolscan_t ** devp)
   return SANE_STATUS_GOOD;
 }
 
+static SANE_Status
+attach_one (const char *devName)
+{
+  return attach_scanner(devName, 0);
+}
+
 static RETSIGTYPE
 sigterm_handler (int signal)
 {
@@ -1426,7 +1442,7 @@ reader_process (Coolscan_t * scanner, int pipe_fd)
   data_left = bytes_per_line (scanner) *
     lines_per_scan (scanner);
 
-  scanner->row_bufsize = sanei_scsi_max_request_size;
+  /*scanner->row_bufsize = sanei_scsi_max_request_size;*/
   coolscan_trim_rowbufsize (scanner);	/* trim bufsize */
 
   DBG (10, "reader_process: reading %u bytes in blocks of %u bytes\n",
@@ -1617,6 +1633,23 @@ init_options (Coolscan_t * scanner)
   scanner->opt[OPT_GAMMA_BIND].type = SANE_TYPE_BOOL;
   scanner->opt[OPT_GAMMA_BIND].unit = SANE_UNIT_NONE;
 
+  
+  scanner->opt[OPT_ANALOG_GAMMA].name = "analog_gamma";
+  scanner->opt[OPT_ANALOG_GAMMA].title = "Analog Gamma";
+  scanner->opt[OPT_ANALOG_GAMMA].desc = "Analog Gamma";
+  scanner->opt[OPT_ANALOG_GAMMA].type = SANE_TYPE_BOOL;
+  scanner->opt[OPT_ANALOG_GAMMA].unit = SANE_UNIT_NONE;
+  if (!scanner->analoggamma)
+    {
+      scanner->opt[OPT_SOURCE].cap = SANE_CAP_INACTIVE;
+    }
+
+  scanner->opt[OPT_AVERAGING].name = "averaging";
+  scanner->opt[OPT_AVERAGING].title = "Averaging";
+  scanner->opt[OPT_AVERAGING].desc = "Averaging";
+  scanner->opt[OPT_AVERAGING].type = SANE_TYPE_BOOL;
+  scanner->opt[OPT_AVERAGING].unit = SANE_UNIT_NONE;
+
 
   scanner->opt[OPT_RGB_CONTROL].name = "rgb-control";
   scanner->opt[OPT_RGB_CONTROL].title = "RGB control";
@@ -1697,12 +1730,21 @@ init_options (Coolscan_t * scanner)
   scanner->opt[OPT_B_CONTRAST].constraint.range = &contrast_range;
   scanner->opt[OPT_B_CONTRAST].cap |= SANE_CAP_INACTIVE;
 
+  scanner->opt[OPT_EXPOSURE].name = "exposure";
+  scanner->opt[OPT_EXPOSURE].title = "Exposure";
+  scanner->opt[OPT_EXPOSURE].desc = "";
+  scanner->opt[OPT_EXPOSURE].type = SANE_TYPE_INT;
+  scanner->opt[OPT_EXPOSURE].cap &= ~SANE_CAP_INACTIVE;
+  scanner->opt[OPT_EXPOSURE].unit = SANE_UNIT_PERCENT;
+  scanner->opt[OPT_EXPOSURE].constraint_type = SANE_CONSTRAINT_RANGE;
+  scanner->opt[OPT_EXPOSURE].constraint.range = &exposure_range;
+
+
   scanner->opt[OPT_R_EXPOSURE].name = "red-exposure";
   scanner->opt[OPT_R_EXPOSURE].title = "Red exposure";
   scanner->opt[OPT_R_EXPOSURE].desc = "";
   scanner->opt[OPT_R_EXPOSURE].type = SANE_TYPE_INT;
-
-  scanner->opt[OPT_R_EXPOSURE].cap &= ~SANE_CAP_INACTIVE;
+  scanner->opt[OPT_R_EXPOSURE].cap |= SANE_CAP_INACTIVE;
   scanner->opt[OPT_R_EXPOSURE].unit = SANE_UNIT_PERCENT;
   scanner->opt[OPT_R_EXPOSURE].constraint_type = SANE_CONSTRAINT_RANGE;
   scanner->opt[OPT_R_EXPOSURE].constraint.range = &exposure_range;
@@ -1714,6 +1756,7 @@ init_options (Coolscan_t * scanner)
   scanner->opt[OPT_G_EXPOSURE].unit = SANE_UNIT_PERCENT;
   scanner->opt[OPT_G_EXPOSURE].constraint_type = SANE_CONSTRAINT_RANGE;
   scanner->opt[OPT_G_EXPOSURE].constraint.range = &exposure_range;
+  scanner->opt[OPT_G_EXPOSURE].cap |= SANE_CAP_INACTIVE;
 
   scanner->opt[OPT_B_EXPOSURE].name = "blue-exposure";
   scanner->opt[OPT_B_EXPOSURE].title = "Blue exposre";
@@ -1722,6 +1765,7 @@ init_options (Coolscan_t * scanner)
   scanner->opt[OPT_B_EXPOSURE].unit = SANE_UNIT_PERCENT;
   scanner->opt[OPT_B_EXPOSURE].constraint_type = SANE_CONSTRAINT_RANGE;
   scanner->opt[OPT_B_EXPOSURE].constraint.range = &exposure_range;
+  scanner->opt[OPT_B_EXPOSURE].cap |= SANE_CAP_INACTIVE;
 
   scanner->opt[OPT_R_SHIFT].name = "red-shift";
   scanner->opt[OPT_R_SHIFT].title = "Red shift";
@@ -1824,9 +1868,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG_INIT ();
   DBG (10, "sane_init\n");
   if (version_code)
-    {
       *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
-    }
 
   fp = sanei_config_open (COOLSCAN_CONFIG_FILE);
   if (!fp)
@@ -1848,7 +1890,8 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
       if (!len)
 	continue;		/* ignore empty lines */
 
-      attach_scanner (dev_name, 0);
+      sanei_config_attach_matching_devices (dev_name, attach_one);
+      /*attach_scanner (dev_name, 0);*/
     }
   fclose (fp);
   return SANE_STATUS_GOOD;
@@ -2085,6 +2128,10 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  *(SANE_Word *) val = scanner->brightness_B - 128;
 	  return SANE_STATUS_GOOD;
 
+	case OPT_EXPOSURE:
+	  *(SANE_Word *) val = scanner->exposure_R * 2;
+	  return SANE_STATUS_GOOD;
+
 	case OPT_R_EXPOSURE:
 	  *(SANE_Word *) val = scanner->exposure_R * 2;
 	  return SANE_STATUS_GOOD;
@@ -2135,6 +2182,15 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  *(SANE_Word *) val = (scanner->gamma_bind) ? SANE_TRUE : SANE_FALSE;
 	  return SANE_STATUS_GOOD;
 
+	case OPT_ANALOG_GAMMA:
+	  *(SANE_Word *) val = 
+	    (scanner->analog_gamma_r) ? SANE_TRUE : SANE_FALSE;
+	  return SANE_STATUS_GOOD;
+
+	case OPT_AVERAGING:
+	  *(SANE_Word *) val = (scanner->averaging) ? SANE_TRUE : SANE_FALSE;
+	  return SANE_STATUS_GOOD;
+
 	case OPT_GAMMA_VECTOR:
 	  memcpy (val, scanner->gamma, scanner->opt[option].size);
 	  return SANE_STATUS_GOOD;
@@ -2207,6 +2263,15 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	    }
 	  return SANE_STATUS_GOOD;
 
+	case OPT_ANALOG_GAMMA:
+	  scanner->analog_gamma_r = scanner->analog_gamma_g =
+	    scanner->analog_gamma_b = (*(SANE_Word *) val == SANE_TRUE);
+	  return SANE_STATUS_GOOD;
+
+	case OPT_AVERAGING:
+	  scanner->averaging = (*(SANE_Word *) val == SANE_TRUE);
+	  return SANE_STATUS_GOOD;
+
 	case OPT_PRESCAN:
 	  scanner->prescan = (*(SANE_Word *) val == SANE_TRUE);
 	  return SANE_STATUS_GOOD;
@@ -2217,6 +2282,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	    {
 	      scanner->opt[OPT_BRIGHTNESS].cap |= SANE_CAP_INACTIVE;
 	      scanner->opt[OPT_CONTRAST].cap |= SANE_CAP_INACTIVE;
+	      scanner->opt[OPT_EXPOSURE].cap |= SANE_CAP_INACTIVE;
 
 	      scanner->opt[OPT_R_BRIGHTNESS].cap &= ~SANE_CAP_INACTIVE;
 	      scanner->opt[OPT_G_BRIGHTNESS].cap &= ~SANE_CAP_INACTIVE;
@@ -2226,20 +2292,31 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	      scanner->opt[OPT_G_CONTRAST].cap &= ~SANE_CAP_INACTIVE;
 	      scanner->opt[OPT_B_CONTRAST].cap &= ~SANE_CAP_INACTIVE;
 
+	      scanner->opt[OPT_R_EXPOSURE].cap &= ~SANE_CAP_INACTIVE;
+	      scanner->opt[OPT_G_EXPOSURE].cap &= ~SANE_CAP_INACTIVE;
+	      scanner->opt[OPT_B_EXPOSURE].cap &= ~SANE_CAP_INACTIVE;
+
 	      scanner->contrast_R = 128;
 	      scanner->contrast_G = 128;
 	      scanner->contrast_B = 128;
 	      scanner->brightness_R = 128;
 	      scanner->brightness_G = 128;
 	      scanner->brightness_B = 128;
+	      scanner->exposure_R = 50;
+	      scanner->exposure_G = 50;
+	      scanner->exposure_B = 50;
 	    }
 	  else
 	    {
 	      scanner->opt[OPT_BRIGHTNESS].cap &= ~SANE_CAP_INACTIVE;
 	      scanner->opt[OPT_CONTRAST].cap &= ~SANE_CAP_INACTIVE;
+	      scanner->opt[OPT_EXPOSURE].cap &= ~SANE_CAP_INACTIVE;
 
 	      scanner->contrast = 128;
 	      scanner->brightness = 128;
+	      scanner->exposure_R = 50;
+	      scanner->exposure_G = 50;
+	      scanner->exposure_B = 50;
 
 	      scanner->opt[OPT_R_BRIGHTNESS].cap |= SANE_CAP_INACTIVE;
 	      scanner->opt[OPT_G_BRIGHTNESS].cap |= SANE_CAP_INACTIVE;
@@ -2248,6 +2325,10 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	      scanner->opt[OPT_R_CONTRAST].cap |= SANE_CAP_INACTIVE;
 	      scanner->opt[OPT_G_CONTRAST].cap |= SANE_CAP_INACTIVE;
 	      scanner->opt[OPT_B_CONTRAST].cap |= SANE_CAP_INACTIVE;
+
+	      scanner->opt[OPT_R_EXPOSURE].cap |= SANE_CAP_INACTIVE;
+	      scanner->opt[OPT_G_EXPOSURE].cap |= SANE_CAP_INACTIVE;
+	      scanner->opt[OPT_B_EXPOSURE].cap |= SANE_CAP_INACTIVE;
 	    }
 	  if (info)
 	    {
@@ -2329,6 +2410,11 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  scanner->brightness_B = *(SANE_Word *) val + 128;
 	  return SANE_STATUS_GOOD;
 
+	case OPT_EXPOSURE:
+	  scanner->exposure_R = *(SANE_Word *) val / 2;
+	  scanner->exposure_G = *(SANE_Word *) val / 2;
+	  scanner->exposure_B = *(SANE_Word *) val / 2;
+	  return SANE_STATUS_GOOD;
 	case OPT_R_EXPOSURE:
 	  scanner->exposure_R = *(SANE_Word *) val / 2;
 	  return SANE_STATUS_GOOD;
@@ -2477,8 +2563,10 @@ sane_start (SANE_Handle handle)
     coolscan_autofocus (scanner);
   else
     {
-      if (scanner->prescan)
-	coolscan_prescan (scanner);
+      if (scanner->prescan) {
+	prescan (scanner);
+	get_internal_info(scanner);
+      }
     }
   /*read_LUT(scanner); */
   send_LUT (scanner);
