@@ -44,7 +44,7 @@
    This file implements a SANE backend for the Polaroid Digital
    Microscope Camera. */
 
-/* $Id: dmc.c,v 1.6 1998/02/26 14:13:42 dskoll Exp $ */
+/* $Id: dmc.c,v 1.13 1998/05/22 18:41:19 dskoll Exp $ */
 
 #include <sane/config.h>
 
@@ -397,6 +397,7 @@ DMCInitOptions(DMC_Camera *c)
 	c->opt[i].unit = SANE_UNIT_NONE;
     }
 
+    c->opt[OPT_NUM_OPTS].name = SANE_NAME_NUM_OPTIONS;
     c->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
     c->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
     c->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
@@ -405,6 +406,7 @@ DMCInitOptions(DMC_Camera *c)
     c->val[OPT_NUM_OPTS].w = NUM_OPTIONS;
 
     c->opt[OPT_GEOMETRY_GROUP].type = SANE_TYPE_GROUP;
+    c->opt[OPT_GEOMETRY_GROUP].name = "";
     c->opt[OPT_GEOMETRY_GROUP].title = "Geometry";
     c->opt[OPT_GEOMETRY_GROUP].desc = "";
     c->opt[OPT_GEOMETRY_GROUP].cap = SANE_CAP_ADVANCED;
@@ -451,6 +453,7 @@ DMCInitOptions(DMC_Camera *c)
     c->val[OPT_BR_Y].w = c->br_y_range.min;
 
     c->opt[OPT_MODE_GROUP].type = SANE_TYPE_GROUP;
+    c->opt[OPT_MODE_GROUP].name = "";
     c->opt[OPT_MODE_GROUP].title = "Imaging Mode";
     c->opt[OPT_MODE_GROUP].desc = "";
     c->opt[OPT_MODE_GROUP].cap = SANE_CAP_ADVANCED;
@@ -770,6 +773,22 @@ DMCReadTwoSuperResolutionLines(DMC_Camera *c, SANE_Byte *buf, int lastLine)
     return SANE_STATUS_GOOD;
 }
 
+// *********************************************************************
+//%FUNCTION: attach_one (static function)
+//%ARGUMENTS:
+// dev -- device to attach
+//%RETURNS:
+// SANE_STATUS_GOOD
+//%DESCRIPTION:
+// tries to attach a device found by sanei_config_attach_matching_devices
+// *********************************************************************
+static SANE_Status
+attach_one (const char *dev)
+{
+  DMCAttach (dev, 0);
+  return SANE_STATUS_GOOD;
+}
+
 /**********************************************************************
 //%FUNCTION: sane_init
 //%ARGUMENTS:
@@ -813,7 +832,7 @@ sane_init(SANE_Int *version_code, SANE_Auth_Callback authorize)
 
 	if (!len) continue;			/* ignore empty lines */
 
-	DMCAttach(dev_name, 0);
+	sanei_config_attach_matching_devices(dev_name, attach_one);
     }
     fclose (fp);
     return SANE_STATUS_GOOD;
@@ -1045,16 +1064,16 @@ sane_control_option(SANE_Handle handle, SANE_Int option,
 	    if (!strcmp(val, ValidModes[i])) {
 		status = DMCSetMode(c, i);
 		c->val[OPT_IMAGE_MODE].s = (SANE_String) ValidModes[i];
-		break;
+		if (info) *info |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
+		return SANE_STATUS_GOOD;
 	    }
 	}
-	*info |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
 	break;
     case OPT_WHITE_BALANCE:
 	for (i=0; i<=WHITE_BALANCE_FLUORESCENT; i++) {
 	    if (!strcmp(val, ValidBalances[i])) {
 		c->val[OPT_WHITE_BALANCE].s = (SANE_String) ValidBalances[i];
-		break;
+		return SANE_STATUS_GOOD;
 	    }
 	}
 	break;
@@ -1062,27 +1081,31 @@ sane_control_option(SANE_Handle handle, SANE_Int option,
 	for (i=1; i<= ASA_100+1; i++) {
 	    if (* ((SANE_Int *) val) == ValidASAs[i]) {
 		c->val[OPT_ASA].w = ValidASAs[i];
-		break;
+		return SANE_STATUS_GOOD;
 	    }
 	}
 	break;
     case OPT_SHUTTER_SPEED:
+	if (* (SANE_Int *) val < c->hw->shutterSpeedRange.min ||
+	    * (SANE_Int *) val > c->hw->shutterSpeedRange.max) {
+	    return SANE_STATUS_INVAL;
+	}
 	c->val[OPT_SHUTTER_SPEED].w = * (SANE_Int *) val;
 	/* Do any roundoff */
 	c->val[OPT_SHUTTER_SPEED].w =
 	    TICKS_TO_MS(MS_TO_TICKS(c->val[OPT_SHUTTER_SPEED].w));
 	if (c->val[OPT_SHUTTER_SPEED].w != * (SANE_Int *) val) {
-	    *info |= SANE_INFO_INEXACT;
+	    if (info) *info |= SANE_INFO_INEXACT;
 	}
 			
-	break;
+	return SANE_STATUS_GOOD;
 
     default:
 	/* Should really be INVAL, but just bit-bucket set requests... */
 	return SANE_STATUS_GOOD;
     }
 
-    return SANE_STATUS_GOOD;
+    return SANE_STATUS_INVAL;
 }
 
 /**********************************************************************
