@@ -60,6 +60,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <ctype.h>
 
 #include <sane/sane.h>
 #include <sane/saneopts.h>
@@ -97,6 +98,9 @@
 #define	EPSON_LEVEL_B1		2
 #define	EPSON_LEVEL_B2		3
 #define	EPSON_LEVEL_B3		4
+#define	EPSON_LEVEL_B4		5
+#define	EPSON_LEVEL_B5		6
+#define	EPSON_LEVEL_B6		7
 
 #define	EPSON_LEVEL_DEFAULT	EPSON_LEVEL_B3
 
@@ -130,6 +134,12 @@ EpsonCmdRec epson_cmd[] =
   {"B2", 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1}
   ,
   {"B3", 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1}
+  ,
+  {"B4", 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1}
+  ,
+  {"B5", 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1}
+  ,
+  {"B6", 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1}
 };
 
 /* TODO: speed */
@@ -147,7 +157,7 @@ scsi_write (int fd, const void *buf, size_t buf_size, SANE_Status * status)
   cmd[4] = buf_size;
   memcpy (cmd + 6, buf, buf_size);
 
-  if (SANE_STATUS_GOOD == (*status = sanei_scsi_cmd (fd, cmd, sizeof (cmd) + buf_size, NULL, NULL)))
+  if (SANE_STATUS_GOOD == (*status = sanei_scsi_cmd (fd, cmd, 6 + buf_size, NULL, NULL)))
     return buf_size;
 
   return 0;
@@ -157,6 +167,17 @@ static int
 send (Epson_Scanner * s, const void *buf, size_t buf_size, SANE_Status * status)
 {
 
+	DBG( 3, "send buf, size = %lu\n", (u_long) buf_size);
+/*
+{
+	size_t k;
+	const char * s = buf;
+
+	for( k = 0; k < buf_size; k++) {
+		DBG( 3, "buf[%u] %02x %c\n", k, s[ k], isprint( s[ k]) ? s[ k] : '.');
+	}
+}
+*/
   if (s->hw->is_scsi)
     {
       return scsi_write (s->fd, buf, buf_size, status);
@@ -196,24 +217,34 @@ scsi_read (int fd, void *buf, size_t buf_size, SANE_Status * status)
 static int 
 receive (Epson_Scanner * s, void *buf, size_t buf_size, SANE_Status * status)
 {
+  int n;
 
   if (s->hw->is_scsi)
     {
-      return scsi_read (s->fd, buf, buf_size, status);
+       n = scsi_read (s->fd, buf, buf_size, status);
     }
   else
     {
-      int n;
 
       if (buf_size == (n = sanei_pio_read (s->fd, buf, buf_size)))
 	*status = SANE_STATUS_GOOD;
       else
 	*status = SANE_STATUS_INVAL;
 
-      return n;
     }
 
-  /* never reached */
+  DBG(3, "receive buf, expected = %lu, got = %d\n", (u_long) buf_size, n);
+/*
+{
+	int k;
+	const char * s = buf;
+
+	for( k = 0; k < n; k++) {
+		DBG( 3, "buf[%u] %02x %c\n", k, s[ k], isprint( s[ k]) ? s[ k] : '.');
+ 	}
+}
+*/
+      return n;
 }
 
 static SANE_Status
@@ -557,7 +588,7 @@ typedef struct
     u_char status;
     u_short count;
 
-    u_char buf[0];
+    u_char buf[1];
 
   }
 EpsonHdrRec, *EpsonHdr;
@@ -571,7 +602,7 @@ typedef struct
     u_char type;
     u_char level;
 
-    u_char buf[0];
+    u_char buf[1];
 
   }
 EpsonIdentRec, *EpsonIdent;
@@ -599,7 +630,11 @@ command (Epson_Scanner * s, const u_char * cmd, size_t cmd_size, SANE_Status * s
     return (EpsonHdr) 0;
 
   buf = (u_char *) head;
-  receive (s, buf, 1, status);
+  if (s->hw->is_scsi)
+    receive (s, buf, 4, status);
+  else
+    receive (s, buf, 1, status);
+
 
   if (SANE_STATUS_GOOD != *status)
     return (EpsonHdr) 0;
@@ -626,13 +661,15 @@ command (Epson_Scanner * s, const u_char * cmd, size_t cmd_size, SANE_Status * s
       break;
 
     case STX:
-      receive (s, buf, 3, status);
+      if (!s->hw->is_scsi)
+        {
+      	  receive (s, buf, 3, status);
+/*	  buf += 3;*/
+	}
 
       if (SANE_STATUS_GOOD != *status)
 	return (EpsonHdr) 0;
 
-/*              buf += 3; */
-
       DBG (4, "status %02x\n", (int) head->status);
       DBG (4, "count  %d\n", (int) head->count);
 
@@ -710,7 +747,8 @@ attach (const char *dev_name, Epson_Device * *devp)
 
       if (buf[0] != TYPE_PROCESSOR
 	  || strncmp (buf + 8, "EPSON", 5) != 0
-	  || strncmp (buf + 16, "SCANNER ", 8) != 0)
+	  || (strncmp (buf + 16, "SCANNER ", 8) != 0
+	      && strncmp (buf + 14, "SCANNER ", 8) != 0))
 	{
 	  DBG (1, "attach: device doesn't look like an Epson scanner\n");
 	  myclose (s);
@@ -731,22 +769,27 @@ attach (const char *dev_name, Epson_Device * *devp)
 
   {
     EpsonIdent ident = (EpsonIdent) command (s, "\033I", 2, &status);
-    u_char *buf = ident->buf;
+    u_char *buf;
     int n, k;
 
-    DBG (1, "type  %3c %02x\n", ident->type, ident->type);
-    DBG (1, "level %3c %02x\n", ident->level, ident->level);
+	if( NULL == ident) {
+		DBG (0, "ident failed\n");
+		return status;
+	}
+
+    DBG (1, "type  %3c 0x%02x\n", ident->type, ident->type);
+    DBG (1, "level %3c 0x%02x\n", ident->level, ident->level);
 
     s->hw->res_list_size = 0;
     s->hw->res_list = (SANE_Int *) calloc (s->hw->res_list_size, sizeof (SANE_Int));
 
     if (NULL == s->hw->res_list)
       {
-	DBG (0, "problem\n");
+	DBG (0, "no mem\n");
 	exit (0);
       }
 
-    for (n = ident->count; n; n -= k, buf += k)
+    for (n = ident->count, buf = ident->buf; n; n -= k, buf += k)
       {
 	switch (*buf)
 	  {
@@ -759,7 +802,7 @@ attach (const char *dev_name, Epson_Device * *devp)
 
 	      if (NULL == s->hw->res_list)
 		{
-		  DBG (0, "problem\n");
+		  DBG (0, "no mem\n");
 		  exit (0);
 		}
 
@@ -808,6 +851,13 @@ attach (const char *dev_name, Epson_Device * *devp)
   return SANE_STATUS_GOOD;
 }
 
+static SANE_Status
+attach_one (const char *dev)
+{
+  attach (dev, 0);
+  return SANE_STATUS_GOOD;
+}
+
 SANE_Status
 sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 {
@@ -843,7 +893,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 	}
       fclose (fp);
     }
-  attach (dev_name, NULL);
+  sanei_config_attach_matching_devices (dev_name, attach_one);
   return SANE_STATUS_GOOD;
 }
 
@@ -1472,17 +1522,21 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
 
 	  receive (s, s->buf + s->params.pixels_per_line, buf_len, &status);
 	  send (s, "\006", 1, &status);
-	  len = 4;		/* TODO, was len = 6 working ? */
+      	  len = s->block ? 6 : 4;
 	  receive (s, result, len, &status);
 	  buf_len = result[3] << 8 | result[2];
+      	  if (s->block)
+	    buf_len *= (result[5] << 8 | result[4]);
 
 	  DBG (5, "sane_read: buf len2 = %lu\n", (u_long) buf_len);
 
 	  receive (s, s->buf, buf_len, &status);
 	  send (s, "\006", 1, &status);
-	  len = 4;		/* TODO, was len = 6 working ? */
+      	  len = s->block ? 6 : 4;
 	  receive (s, result, len, &status);
 	  buf_len = result[3] << 8 | result[2];
+      	  if (s->block)
+	    buf_len *= (result[5] << 8 | result[4]);
 
 	  DBG (5, "sane_read: buf len3 = %lu\n", (u_long) buf_len);
 
