@@ -200,10 +200,10 @@ static SANE_Status
 attach (const char *devname, HP_Device **devp)
 {
   char result[INQ_LEN], *str, *t;
-  int i, fd, hp_scanner;
   SANE_Status status;
   HP_Device *dev;
   size_t size;
+  int i, fd;
   struct
     {
       int code;
@@ -261,6 +261,15 @@ attach (const char *devname, HP_Device **devp)
       return status;
     }
 
+  /* are we really dealing with a scanner by HP? */
+  if (!((result[0] == 0x03 || result[0] == 0x06)
+	&& strncmp (result + 8, "HP", 2) == 0))
+    {
+      DBG(1, "attach: device doesn't look like a HP scanner\n");
+      sanei_scsi_close (fd);
+      return SANE_STATUS_INVAL;
+    }
+
   DBG(3, "attach: sending TEST_UNIT_READY\n");
   status = sanei_scsi_cmd (fd, test_unit_ready, sizeof (test_unit_ready),
 			   0, 0);
@@ -283,15 +292,6 @@ attach (const char *devname, HP_Device **devp)
 
   sanei_scsi_close (fd);
 
-  /* are we really dealing with a scanner by HP? */
-  hp_scanner = ((result[0] == 0x03 || result[0] == 0x06)
-		&& strncmp (result + 8, "HP", 2) == 0);
-  if (!hp_scanner)
-    {
-      DBG(1, "attach: device doesn't look like a HP scanner\n");
-      return SANE_STATUS_INVAL;
-    }
-
   dev = malloc (sizeof (*dev));
   if (!dev)
     return SANE_STATUS_NO_MEM;
@@ -495,6 +495,13 @@ do_cancel (HP_Scanner *s)
   return SANE_STATUS_CANCELLED;
 }
 
+static SANE_Status
+attach_one (const char *dev)
+{
+  attach (dev, 0);
+  return SANE_STATUS_GOOD;
+}
+
 SANE_Status
 sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
 {
@@ -520,13 +527,13 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
       if (dev_name[0] == '#')		/* ignore line comments */
 	continue;
       len = strlen (dev_name);
-      if (dev_name[len - 1] == '\n')
+      if (len > 0 && dev_name[len - 1] == '\n')
 	dev_name[--len] = '\0';
 
       if (!len)
 	continue;			/* ignore empty lines */
 
-      attach (dev_name, 0);
+      sanei_config_attach_matching_devices (dev_name, attach_one);
     }
   fclose (fp);
   return SANE_STATUS_GOOD;
