@@ -93,6 +93,10 @@ static int num_devices;
 static Mustek_Device *first_dev;
 static Mustek_Scanner *first_handle;
 
+static Mustek_Device **new_dev;	/* array of newly attached devices */
+static int new_dev_len;		/* length of new_dev array */
+static int new_dev_alloced;	/* number of entries alloced for new_dev */
+
 /* Used for line-distance correction: */
 static const int color_seq[] =
   {
@@ -444,6 +448,9 @@ attach (const char *devname, Mustek_Device **devp, int may_wait)
   SANE_Status status;
   size_t size;
 
+  if (devp)
+    *devp = 0;
+
   for (dev = first_dev; dev; dev = dev->next)
     if (strcmp (dev->sane.name, devname) == 0)
       {
@@ -476,11 +483,6 @@ attach (const char *devname, Mustek_Device **devp, int may_wait)
       return status;
     }
 
-  status = dev_wait_ready (&s);
-  dev_close (&s);
-  if (status != SANE_STATUS_GOOD)
-    return status;
-
   /* first check for new firmware format: */
   mustek_scanner = (strncmp (result + 36, "MUSTEK", 6) == 0);
   if (!mustek_scanner)
@@ -495,9 +497,15 @@ attach (const char *devname, Mustek_Device **devp, int may_wait)
     {
       DBG(1, "attach: device doesn't look like a Mustek scanner "
 	  "(result[0]=%#02x)\n", result[0]);
+      dev_close (&s);
       return SANE_STATUS_INVAL;
     }
 
+  status = dev_wait_ready (&s);
+  dev_close (&s);
+  if (status != SANE_STATUS_GOOD)
+    return status;
+
   /* get firmware revision as BCD number: */
   fw_revision =
       (result[32] - '0') << 8 | (result[34] - '0') << 4 | (result[35] - '0');
@@ -2017,14 +2025,41 @@ reader_process (Mustek_Scanner *s, int fd)
   return SANE_STATUS_GOOD;
 }
 
+static SANE_Status
+attach_one_device (const char *devname)
+{
+  Mustek_Device *dev;
+
+  attach (devname, &dev, SANE_FALSE);
+  if (dev)
+    {
+      /* Keep track of newly attached devices so we can set options as
+	 necessary.  */
+      if (new_dev_len >= new_dev_alloced)
+	{
+	  new_dev_alloced += 4;
+	  if (new_dev)
+	    new_dev = realloc (new_dev, new_dev_alloced * sizeof (new_dev[0]));
+	  else
+	    new_dev = malloc (new_dev_alloced * sizeof (new_dev[0]));
+	  if (!new_dev)
+	    {
+	      DBG (1, "attach_one_device: out of memory\n");
+	      return SANE_STATUS_NO_MEM;
+	    }
+	}
+      new_dev[new_dev_len++] = dev;
+    }
+  return SANE_STATUS_GOOD;
+}
+
 SANE_Status
 sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
 {
-  static char prev_dev_name[PATH_MAX] = "";
   char dev_name[PATH_MAX], *cp;
-  Mustek_Device *dev;
   size_t len;
   FILE *fp;
+  int i;
 
   DBG_INIT();
 
@@ -2041,10 +2076,8 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
 
   while (fgets (dev_name, sizeof (dev_name), fp))
     {
-      /* skip white space: */
-      for (cp = dev_name; isspace(*cp); ++cp);
-
-      if (*cp == '#')		/* ignore line comments */
+      cp = (char *) sanei_config_skip_whitespace (dev_name);
+      if (!*cp || *cp == '#')	/* ignore line comments & empty lines */
 	continue;
 
       len = strlen (cp);
@@ -2057,8 +2090,7 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
       if (strncmp (cp, "option", 6) == 0 && isspace (cp[6]))
 	{
 	  cp += 7;
-
-	  while (isspace (*cp)) ++cp;
+	  cp = (char *) sanei_config_skip_whitespace (cp);
 
 	  if (strncmp (cp, "strip-height", 12) == 0 && isspace (cp[12]))
 	    {
@@ -2083,36 +2115,37 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
 	    }
 	  else if (strncmp (cp, "linedistance-fix", 16) == 0 &&
 		   (isspace (cp[16]) || !cp[16]))
-	    for (dev = first_dev; dev; dev = dev->next)
+	    for (i = 0; i < new_dev_len; ++i)
 	      {
-		if (strcmp (dev->sane.name, prev_dev_name) == 0)
-		  {
-		    dev->flags |= MUSTEK_FLAG_LD_FIX;
-		    DBG(2, "sane_init: enabling linedistance-fix for %s\n",
-			dev->sane.name);
-		    break;
-		  }
+		new_dev[i]->flags |= MUSTEK_FLAG_LD_FIX;
+		DBG(2, "sane_init: enabling linedistance-fix for %s\n",
+		    new_dev[i]->sane.name);
+		break;
 	      }
 	  else if (strncmp (cp, "lineart-fix", 11) == 0 &&
 		   (isspace (cp[11]) || !cp[11]))
-	    for (dev = first_dev; dev; dev = dev->next)
+	    for (i = 0; i < new_dev_len; ++i)
 	      {
-		if (strcmp (dev->sane.name, prev_dev_name) == 0)
-		  {
-		    dev->flags |= MUSTEK_FLAG_LINEART_FIX;
-		    DBG(2, "sane_init: enabling lineart-fix for %s\n",
-			dev->sane.name);
-		    break;
-		  }
+		new_dev[i]->flags |= MUSTEK_FLAG_LINEART_FIX;
+		DBG(2, "sane_init: enabling lineart-fix for %s\n",
+		    new_dev[i]->sane.name);
+		break;
 	      }
 	  else
 	    DBG(1, "%s: ignoring unknown option `%s'\n",
 		MUSTEK_CONFIG_FILE, cp);
 	  continue;
 	}
-
-      attach (cp, 0, SANE_FALSE);
-      strcpy (prev_dev_name, cp);
+      else
+	{
+	  new_dev_len = 0;
+	  sanei_config_attach_matching_devices (cp, attach_one_device);
+	}
+    }
+  if (new_dev_alloced > 0)
+    {
+      new_dev_len = new_dev_alloced = 0;
+      free (new_dev);
     }
   fclose (fp);
   return SANE_STATUS_GOOD;
@@ -2683,9 +2716,6 @@ sane_start (SANE_Handle handle)
   if (status != SANE_STATUS_GOOD)
     goto stop_scanner_and_return;
 
-  if (status != SANE_STATUS_GOOD)
-    goto stop_scanner_and_return;
-
   s->ld.max_value = 0;
   if (s->hw->flags & MUSTEK_FLAG_SINGLE_PASS)
     {
