@@ -308,14 +308,14 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   if (version_code)
     *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
 
-  serv = getservbyname ("saned", "tcp");
+  serv = getservbyname ("sane", "tcp");
   if (serv)
     saned_port = serv->s_port;
   else
     {
       saned_port = htons (6566);
       DBG(1,
-	  "init: could not find `saned' service (%s); using default port %d\n",
+	  "init: could not find `sane' service (%s); using default port %d\n",
 	  strerror (errno), htons (saned_port));
     }
 
