@@ -809,6 +809,12 @@ init_options (S9036_Scanner * s)
   return SANE_STATUS_GOOD;
 }
 
+static SANE_Status
+attach_one (const char *dev)
+{
+  attach (dev, 0);
+  return SANE_STATUS_GOOD;
+}
 
 SANE_Status
 sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
@@ -841,7 +847,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
       if (!len)
 	continue;		/* ignore empty lines */
 
-      attach (dev_name, 0);
+      sanei_config_attach_matching_devices (dev_name, attach_one);
     }
   fclose (fp);
   return SANE_STATUS_GOOD;
