@@ -866,6 +866,14 @@ reader_process (Tamarack_Scanner *s, int fd)
 }
 
 
+static SANE_Status
+attach_one (const char *dev)
+{
+  attach (dev, 0);
+  return SANE_STATUS_GOOD;
+}
+
+
 SANE_Status
 sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
 {
@@ -895,7 +903,7 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
     if (!len)
       continue;			/* ignore empty lines */
     
-    attach (dev_name, 0);
+    sanei_config_attach_matching_devices (dev_name, attach_one);
   }
   fclose (fp);
   return SANE_STATUS_GOOD;
