@@ -255,7 +255,7 @@ static scsiblk inquiry = { inquiryC, sizeof(inquiryC) };
 
 #define get_inquiry_gamma_download_available(in)		getbitfield(in + 0x64, 1, 7)
 #define get_inquiry_0x64_bit6(in)				getbitfield(in + 0x64, 1, 6)
-#define get_inquiry_gamma_type(in)				getbitfield(in + 0x64, 1, 5)
+#define get_inquiry_gamma_type_2(in)				getbitfield(in + 0x64, 1, 5)
 #define get_inquiry_0x64_bit4(in)				getbitfield(in + 0x64, 1, 4)
 #define get_inquiry_0x64_bit3(in)				getbitfield(in + 0x64, 1, 3)
 #define get_inquiry_0x64_bit2(in)				getbitfield(in + 0x64, 1, 2)
@@ -267,11 +267,11 @@ static scsiblk inquiry = { inquiryC, sizeof(inquiryC) };
 
 #define get_inquiry_gib(in)					in[0x66]
 #define get_inquiry_gib_8bpp(in)				getbitfield(in + 0x66, 1, 0) 
-#define get_inquiry_0x66_bit1(in)				getbitfield(in + 0x66, 1, 1)
+#define get_inquiry_gib_9bpp(in)				getbitfield(in + 0x66, 1, 1)
 #define get_inquiry_gib_10bpp(in)				getbitfield(in + 0x66, 1, 2) 
-#define get_inquiry_0x66_bit3(in)				getbitfield(in + 0x66, 1, 3)
-#define get_inquiry_0x66_bit4(in)				getbitfield(in + 0x66, 1, 4)
-#define get_inquiry_0x66_bit5(in)				getbitfield(in + 0x66, 1, 5)
+#define get_inquiry_gib_12bpp(in)				getbitfield(in + 0x66, 1, 3)
+#define get_inquiry_gib_14bpp(in)				getbitfield(in + 0x66, 1, 4)
+#define get_inquiry_gib_16bpp(in)				getbitfield(in + 0x66, 1, 5)
 #define get_inquiry_0x66_bit6(in)				getbitfield(in + 0x66, 1, 6)
 #define get_inquiry_0x66_bit7(in)				getbitfield(in + 0x66, 1, 7)
 #  define IN_gib_8bpp			0x01
@@ -281,11 +281,11 @@ static scsiblk inquiry = { inquiryC, sizeof(inquiryC) };
 
 #define get_inquiry_gob(in)					in[0x68]
 #define get_inquiry_gob_8bpp(in)				getbitfield(in + 0x68, 1, 0) 
-#define get_inquiry_0x68_bit1(in)				getbitfield(in + 0x68, 1, 1)
+#define get_inquiry_gob_9bpp(in)				getbitfield(in + 0x68, 1, 1)
 #define get_inquiry_gob_10bpp(in)				getbitfield(in + 0x68, 1, 2) 
-#define get_inquiry_0x68_bit3(in)				getbitfield(in + 0x68, 1, 3)
-#define get_inquiry_0x68_bit4(in)				getbitfield(in + 0x68, 1, 4)
-#define get_inquiry_0x68_bit5(in)				getbitfield(in + 0x68, 1, 5)
+#define get_inquiry_gob_12bpp(in)				getbitfield(in + 0x68, 1, 3)
+#define get_inquiry_gob_14bpp(in)				getbitfield(in + 0x68, 1, 4)
+#define get_inquiry_gob_16bpp(in)				getbitfield(in + 0x68, 1, 5)
 #define get_inquiry_0x68_bit6(in)				getbitfield(in + 0x68, 1, 6)
 #define get_inquiry_0x68_bit7(in)				getbitfield(in + 0x68, 1, 7)
 #  define IN_gob_8bpp			0x01
@@ -746,8 +746,29 @@ static scsiblk send = { sendC, sizeof(sendC) };
 /* FIXME: add Type 1 download curve format */
 
 
-static unsigned char gamma_DCF2C[] = { 0x21, 0x00, 0x01, 0x00, 0x01, 0x00 };
+static unsigned char gamma_DCF0C[] = { 0x00 };
+static scsiblk gamma_DCF0 = { gamma_DCF0C, sizeof(gamma_DCF0C) };
+#define set_DCF0_gamma_color(dc,col,val)		dc[1 + (1025 * col)] = val
+#  define DCF0_gamma_color_grey		0
+#  define DCF0_gamma_color_red		1
+#  define DCF0_gamma_color_green	2
+#  define DCF0_gamma_color_blue		3
 
+#define set_DCF0_gamma_lines(dc,val)			setbitfield(dc + 0x00, 3, 0, val)
+#  define DCF0_gamma_one_line		1
+#  define DCF0_gamma_three_lines	3
+
+
+static unsigned char gamma_DCF1C[] = { 0x00, 0x00 };
+static scsiblk gamma_DCF1 = { gamma_DCF1C, sizeof(gamma_DCF1C) };
+#define set_DCF1_gamma_color(dc,val)			dc[1] = val
+#  define DCF1_gamma_color_grey		0
+#  define DCF1_gamma_color_red		1
+#  define DCF1_gamma_color_green	2
+#  define DCF1_gamma_color_blue		3
+
+
+static unsigned char gamma_DCF2C[] = { 0x21, 0x00, 0x01, 0x00, 0x01, 0x00 };
 static scsiblk gamma_DCF2 = { gamma_DCF2C, sizeof(gamma_DCF2C) };
 
 #define set_DCF2_gamma_color(dc,val)			setbitfield(dc + 0x00, 3, 2, val)
