@@ -206,10 +206,9 @@ set_option (GSGDialog * dialog, int opt_num, void *val, SANE_Action action)
 }
 
 void
-gsg_close_dialog_callback (GtkWidget * widget,
-			   gpointer client_data, gpointer call_data)
+gsg_close_dialog_callback (GtkWidget * widget, gpointer data)
 {
-  gtk_widget_destroy (client_data);
+  gtk_widget_destroy (data);
   gsg_message_dialog_active = 0;
 }
 
@@ -293,7 +292,7 @@ gsg_get_filename (const char *label, const char *default_name,
 
   while (!cancel && !ok)
     {
-      if (!gdk_events_pending ())
+      if (!gtk_events_pending ())
 	usleep (100000);
       gtk_main_iteration ();
     }
