@@ -131,11 +131,6 @@ draw_selection (Preview *p)
     /* window isn't mapped yet */
     return;
 
-#if 0
-  if (memcmp(&p->selection, &p->previous_selection, sizeof(p->selection)) == 0)
-    return;		/* no change */
-#endif
-
   if (p->previous_selection.active)
     draw_rect (p->window->window, p->gc, p->previous_selection.coord);
 
@@ -143,9 +138,6 @@ draw_selection (Preview *p)
     draw_rect (p->window->window, p->gc, p->selection.coord);
 
   p->previous_selection = p->selection;
-#if 0
-  update_selection (p);
-#endif
 }
 
 static void
@@ -674,7 +666,7 @@ input_available (gpointer data, gint source, GdkInputCondition cond)
       if (p->input_tag < 0)
 	{
 	  display_maybe (p);
-	  while (gdk_events_pending ())
+	  while (gtk_events_pending ())
 	    gtk_main_iteration ();
 	}
     }
@@ -1254,6 +1246,7 @@ preview_update (Preview *p)
       if (preferences.preserve_preview)
 	restore_preview_image (p);
     }
+  update_selection (p);
 }
 
 void
