@@ -901,13 +901,13 @@ main (int argc, char *argv[])
 
       memset (&sin, 0, sizeof (sin));
 
-      serv = getservbyname ("saned", "tcp");
+      serv = getservbyname ("sane", "tcp");
       if (serv)
 	port = serv->s_port;
       else
 	{
 	  port = htons (6566);
-	  fprintf (stderr, "%s: could not find `saned' service (%s)\n"
+	  fprintf (stderr, "%s: could not find `sane' service (%s)\n"
 		   "%s: using default port %d\n", prog_name, strerror (errno),
 		   prog_name, ntohs (port));
 	}
