@@ -446,7 +446,7 @@ quit_xscan (void)
     }
   while (gsg_message_dialog_active)
     {
-      if (!gdk_events_pending ())
+      if (!gtk_events_pending ())
 	usleep (100000);
       gtk_main_iteration ();
     }
@@ -465,14 +465,14 @@ quit_xscan (void)
 /* Invoked when window manager's "delete" (or "close") function is
    invoked.  */
 static gint
-scan_win_delete (GtkWidget *w, GdkEvent *e, gpointer data)
+scan_win_delete (GtkWidget *w, gpointer data)
 {
   quit_xscan();
   return FALSE;
 }
 
 static void
-preview_window_destroyed (GtkWidget *widget, GdkEvent *e, gpointer call_data)
+preview_window_destroyed (GtkWidget *widget, gpointer call_data)
 {
   gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (call_data), FALSE);
 }
@@ -488,16 +488,10 @@ scan_preview (GtkWidget * widget, gpointer call_data)
 	{
 	  scan_win.preview = preview_new (dialog);
 	  if (scan_win.preview && scan_win.preview->top)
-	    {
-	      gtk_signal_connect (GTK_OBJECT (scan_win.preview->top),
-				  "destroy",
-				  GTK_SIGNAL_FUNC (preview_window_destroyed),
-				  widget);
-	      gtk_signal_connect (GTK_OBJECT (scan_win.preview->top),
-				  "delete_event",
-				  GTK_SIGNAL_FUNC (preview_window_destroyed),
-				  widget);
-	    }
+	    gtk_signal_connect (GTK_OBJECT (scan_win.preview->top),
+				"destroy",
+				GTK_SIGNAL_FUNC (preview_window_destroyed),
+				widget);
 	  else
 	    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (widget), FALSE);
 	}
@@ -587,7 +581,7 @@ input_available (gpointer data, gint source, GdkInputCondition cond)
       progress_update (scan_win.progress,
 		       scan_win.bytes_read / (gfloat) scan_win.num_bytes);
       if (scan_win.input_tag < 0)
-	while (gdk_events_pending ())
+	while (gtk_events_pending ())
 	  gtk_main_iteration ();
 
       switch (scan_win.param.format)
@@ -734,12 +728,14 @@ input_available (gpointer data, gint source, GdkInputCondition cond)
     }
   return;
 
+#ifdef HAVE_LIBGIMP_GIMP_H
 bad_depth:
   snprintf (buf, sizeof (buf), "Cannot handle depth %d.",
 	    scan_win.param.depth);
   gsg_error (buf);
   scan_done ();
   return;
+#endif
 }
 
 static void
@@ -949,7 +945,7 @@ scan_start (void)
   if (scan_win.progress)
     progress_free (scan_win.progress);
   scan_win.progress = progress_new ("Scanning", buf,
-				   (GtkSignalFunc) progress_cancel, 0);
+				    (GtkSignalFunc) progress_cancel, 0);
 
   scan_win.input_tag = -1;
   if (sane_set_io_mode (dev, SANE_TRUE) == SANE_STATUS_GOOD
@@ -962,11 +958,9 @@ scan_start (void)
 
 /* Invoked when the scan button is pressed */
 static void
-scan_dialog (GtkWidget * widget,
-	     gpointer client_data, gpointer call_data)
+scan_dialog (GtkWidget * widget, gpointer call_data)
 {
   char buf[256];
-
   
   if (scan_win.mode == STANDALONE)
     {				/* We are running in standalone mode */
@@ -1623,12 +1617,12 @@ init (int argc, char **argv)
       static const SANE_Device *device_list[] = { &dev, 0 };
       int ch;
 
-      while ((ch = getopt_long (argc, argv, "hV", long_options, 0)) != EOF)
+      while ((ch = getopt_long (argc, argv, "ghV", long_options, 0)) != EOF)
 	{
 	  switch (ch)
 	    {
 	    case 'g':
-	      printf ("%s: GIMP support missing.", argv[0]);
+	      printf ("%s: GIMP support missing.\n", argv[0]);
 	      exit (0);
 
 	    case 'V':
