@@ -41,13 +41,14 @@
    This file provides a generic SCSI interface.  */
 
 #ifdef _AIX
-#include <lalloca.h>		/* MUST come first for AIX! */
+# include <lalloca.h>		/* MUST come first for AIX! */
 #endif
 
 #include <sane/config.h>
 #include <lalloca.h>
 
 #include <assert.h>
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdlib.h>
@@ -147,8 +148,9 @@
 # define USE STUBBED_INTERFACE
 #endif
 
-#include <sane/sanei_scsi.h>
 #include <sane/sanei.h>
+#include <sane/sanei_config.h>
+#include <sane/sanei_scsi.h>
 
 #define BACKEND_NAME	sanei_scsi
 #include <sane/sanei_debug.h>
@@ -216,7 +218,7 @@ long CommandTriggerValue[2];
 ec2_$ptr_t CommandAcceptedPtr[2];
 long ResultTriggerValue[2];
 ec2_$ptr_t ResultReadyPtr[2];
-time_$clock_t Wait16S = {64, 0};		/* Delay of about 16 Seconds */
+time_$clock_t Wait16S = { 64, 0 };		/* Delay of about 16 Seconds */
 
 
 /* This function is registered as an exit function.  It's purpose is
@@ -224,7 +226,7 @@ time_$clock_t Wait16S = {64, 0};		/* Delay of about 16 Seconds */
    send an Exit command, and if that fails, it will send SIGQUIT to
    the server.  It will also unmap the common area before it
    returns. */
-static void 
+static void
 KillDomainServer (void)
 {
   static boolean GotTheLock;
@@ -242,7 +244,7 @@ KillDomainServer (void)
       DomainErrorCheck (status, "Can't advance CommandAvailable EC");
       /* For this wait, we want to allow a timeout as well */
       CommandTriggerValue[1] = (ec2_$read (*CommandAcceptedPtr[1])
-				+ DomainECWaitConstant;
+				+ DomainECWaitConstant);
       index = ec2_$wait_svc (CommandAcceptedPtr, CommandTriggerValue, 2,
 			     &status);
       DomainErrorCheck (status, "Error waiting on Exit command acceptance EC");
@@ -273,10 +275,21 @@ KillDomainServer (void)
 #if USE == OS2_INTERFACE
 
 /* Driver info:  */
-static HEV postSema = 0;    /* Event Semaphore for posting SRB completion */
+static HEV postSema = 0; /* Event Semaphore for posting SRB completion */
 static HFILE driver_handle = 0;	/* file handle for device driver */
 static PVOID aspi_buf = 0;	/* Big data buffer locked by driver. */
 static int aspi_ref_count = 0;	/* # of fds using ASPI */
+static char tmpAspi[MAXPATHLEN];	/* scsi chain scan */
+#define INQUIRY					0x12
+#define set_inquiry_return_size(icb,val)	icb[0x04]=val
+#define IN_periph_devtype_cpu			0x03
+#define IN_periph_devtype_scanner		0x06
+#define get_inquiry_vendor(in, buf)		strncpy(buf, in + 0x08, 0x08)
+#define get_inquiry_product(in, buf)		strncpy(buf, in + 0x10, 0x010)
+#define get_inquiry_version(in, buf)		strncpy(buf, in + 0x20, 0x04)
+#define get_inquiry_periph_devtype(in)		(in[0] & 0x1f)
+#define get_inquiry_additional_length(in)	in[0x04]
+#define set_inquiry_length(out,n)		out[0x04]=n-5
 
 /* Open OS2 ASPI driver.
 
@@ -290,6 +303,17 @@ open_aspi (void)
   USHORT lockSegmentReturn;
   unsigned long cbreturn;
   unsigned long cbParam;
+  SRB SRBlock;			/* SCSI Request Block */
+
+  int i, num_adapters;		/* no. of scsi adapters installed */
+
+  char *devtypes[] =
+  {
+    "disk", "tape", "printer", "processor", "CD-writer",
+    "CD-drive", "scanner", "optical-drive", "jukebox",
+    "communicator"
+  };
+  FILE *tmp;
 
   if (driver_handle)
     return 1;			/* Already open. */
@@ -343,6 +367,172 @@ open_aspi (void)
       DBG (1, "open_aspi:  Can't lock buffer.\n");
       return 0;
     }
+
+  /* query number of installed adapters */
+  memset (&SRBlock, 0, sizeof (SRBlock));
+  SRBlock.cmd = SRB_Inquiry;	/* host adapter inquiry */
+
+  SRBlock.ha_num = 0;		/* host adapter number */
+
+  SRBlock.flags = 0;		/* no flags set */
+
+  rc = DosDevIOCtl (driver_handle, 0x92, 0x02,
+		    (void *) &SRBlock, sizeof (SRB), &cbParam,
+		    (void *) &SRBlock, sizeof (SRB), &cbreturn);
+  num_adapters = SRBlock.u.inq.num_ha;
+
+  DBG (1, "OS/2: installed adapters %d\n", num_adapters);
+  DBG (1, "OS/2: ASPI manager is '%s'\n", SRBlock.u.inq.aspimgr_id);
+  DBG (1, "OS/2: host adapter is '%s'\n", SRBlock.u.inq.host_id);
+  DBG (1, "OS/2: unique id is    '%s'\n", SRBlock.u.inq.unique_id);
+
+  strcpy (tmpAspi, "asXXXXXX");
+  mktemp (tmpAspi);
+  DBG (1, "open_aspi: temporary file is '%s'\n", tmpAspi);
+  tmp = fopen (tmpAspi, "w");
+  if (!tmp)
+    {				/* can't open tmp file */
+
+      DBG (1, "open_aspi:  Can't open temporary file.\n");
+      return 0;
+    }
+
+  /* scan all installed adapters */
+  for (i = 0; i < num_adapters; i++)
+    {
+      int id;
+      /* query adapter name */
+      memset (&SRBlock, 0, sizeof (SRBlock));
+      SRBlock.cmd = SRB_Inquiry;	/* host adapter inquiry */
+
+      SRBlock.ha_num = i;	/* host adapter number */
+
+      SRBlock.flags = 0;	/* no flags set */
+
+      rc = DosDevIOCtl (driver_handle, 0x92, 0x02,
+			(void *) &SRBlock, sizeof (SRB), &cbParam,
+			(void *) &SRBlock, sizeof (SRB), &cbreturn);
+      DBG (1, "OS/2: adapter#%02d '%s'\n", i, SRBlock.u.inq.host_id);
+
+      /* scan scsi chain (need 15 for wide?) */
+      for (id = 0; id < 7; id++)
+	{
+	  unsigned char len;
+	  unsigned char vendor[9];
+	  unsigned char product[17];
+	  unsigned char version[5];
+	  unsigned char *pp;
+
+	  memset (&SRBlock, 0, sizeof (SRBlock));
+	  SRBlock.cmd = SRB_Device;	/* get device type */
+
+	  SRBlock.ha_num = i;	/* host adapter number */
+
+	  SRBlock.flags = 0;	/* no flags set */
+
+	  SRBlock.u.dev.target = id;	/* target id */
+
+	  SRBlock.u.dev.lun = 0;	/* target LUN */
+
+	  rc = DosDevIOCtl (driver_handle, 0x92, 0x02,
+			    (void *) &SRBlock, sizeof (SRB), &cbParam,
+			    (void *) &SRBlock, sizeof (SRB), &cbreturn);
+	  DBG (1, "OS/2:             id#%02d status=%02xh\n",
+	       id, SRBlock.status);
+
+	  /* skip if device not connected */
+	  if (SRBlock.status == SRB_BadDevice)
+	    continue;
+
+	  DBG (1, "OS/2:                   type is '%s'\n",
+	       SRBlock.u.dev.devtype < sizeof (devtypes) / sizeof (char *)?
+	       devtypes[SRBlock.u.dev.devtype] : "unknown device");
+
+	  /* query adapter string id */
+	  memset (&SRBlock, 0, sizeof (SRBlock));
+	  SRBlock.cmd = SRB_Command;	/* execute SCSI command */
+
+	  SRBlock.ha_num = i;	/* host adapter number */
+	  SRBlock.flags = SRB_Read | SRB_Post;	/* data transfer, posting */
+	  SRBlock.u.cmd.target = id;	/* Target SCSI ID */
+	  SRBlock.u.cmd.lun = 0;	/* Target SCSI LUN */
+	  SRBlock.u.cmd.data_len = 5;	/* # of bytes transferred */
+	  SRBlock.u.cmd.sense_len = 16;		/* length of sense buffer */
+	  SRBlock.u.cmd.data_ptr = NULL;	/* pointer to data buffer */
+	  SRBlock.u.cmd.link_ptr = NULL;	/* pointer to next SRB */
+	  SRBlock.u.cmd.cdb_len = 6;	/* SCSI command length */
+	  SRBlock.u.cmd.cdb_st[0] = INQUIRY;	/* inquiry command */
+	  SRBlock.u.cmd.cdb_st[1] = 0;	/* ?? length */
+	  SRBlock.u.cmd.cdb_st[2] = 0;	/* transfer length MSB */
+	  SRBlock.u.cmd.cdb_st[3] = 0;	/* transfer length */
+	  SRBlock.u.cmd.cdb_st[4] = 5;	/* transfer length LSB */
+	  SRBlock.u.cmd.cdb_st[5] = 0;
+	  rc = DosDevIOCtl (driver_handle, 0x92, 0x02,
+			    (void *) &SRBlock, sizeof (SRB), &cbParam,
+			    (void *) &SRBlock, sizeof (SRB), &cbreturn);
+	  DosWaitEventSem (postSema, -1);
+	  DosResetEventSem (postSema, &rc);
+	  len = ((char *) aspi_buf)[4];		/* additional length */
+
+	  /* query id string */
+	  memset (&SRBlock, 0, sizeof (SRBlock));
+	  SRBlock.cmd = SRB_Command;	/* execute SCSI command */
+	  SRBlock.ha_num = i;	/* host adapter number */
+	  SRBlock.flags = SRB_Read | SRB_Post;	/* data transfer, posting */
+	  SRBlock.u.cmd.target = id;	/* Target SCSI ID */
+	  SRBlock.u.cmd.lun = 0;	/* Target SCSI LUN */
+	  SRBlock.u.cmd.data_len = 5 + len;	/* # of bytes transferred */
+	  SRBlock.u.cmd.sense_len = 16;		/* length of sense buffer */
+	  SRBlock.u.cmd.data_ptr = NULL;	/* pointer to data buffer */
+	  SRBlock.u.cmd.link_ptr = NULL;	/* pointer to next SRB */
+	  SRBlock.u.cmd.cdb_len = 6;	/* SCSI command length */
+	  SRBlock.u.cmd.cdb_st[0] = 0x12;	/* inquiry command */
+	  SRBlock.u.cmd.cdb_st[1] = 0;	/* ?? length */
+	  SRBlock.u.cmd.cdb_st[2] = 0;	/* transfer length MSB */
+	  SRBlock.u.cmd.cdb_st[3] = 0;	/* transfer length */
+	  SRBlock.u.cmd.cdb_st[4] = 5 + len;	/* transfer length LSB */
+	  SRBlock.u.cmd.cdb_st[5] = 0;
+	  rc = DosDevIOCtl (driver_handle, 0x92, 0x02,
+			    (void *) &SRBlock, sizeof (SRB), &cbParam,
+			    (void *) &SRBlock, sizeof (SRB), &cbreturn);
+	  DosWaitEventSem (postSema, -1);
+	  DosResetEventSem (postSema, &rc);
+	  DBG (1, "OS/2         '%s'\n", (char *) aspi_buf + 8);
+	  /* write data */
+	  get_inquiry_vendor ((char *) aspi_buf, vendor);
+	  get_inquiry_product ((char *) aspi_buf, product);
+	  get_inquiry_version ((char *) aspi_buf, version);
+
+	  pp = &vendor[7];
+	  vendor[8] = '\0';
+	  while (pp >= vendor && (*pp == ' ' || *pp >= 127))
+	    *pp-- = '\0';
+
+	  pp = &product[15];
+	  product[16] = '\0';
+	  while (pp >= product && (*pp == ' ' || *pp >= 127))
+	    *pp-- = '\0';
+
+	  pp = product;
+	  do
+	    {
+	      if (isspace (*pp))
+		*pp = '_';
+	    }
+	  while (*++pp);
+
+	  pp = &version[3];
+	  version[4] = '\0';
+	  while (pp >= version && (*pp == ' ' || *(pp - 1) >= 127))
+	    *pp-- = '\0';
+	  fprintf (tmp, "Vendor: %s ", vendor);
+	  fprintf (tmp, "Model: %s ", product);
+	  fprintf (tmp, "Rev: %s ", version);
+	  fprintf (tmp, "scsi %d Channel: 0 Id: %d Lun: 0\n", i, id);
+	}
+    }
+  fclose (tmp);
+
   return 1;
 }
 
@@ -360,6 +550,11 @@ close_aspi (void)
   if (aspi_buf)			/* Free buffer. */
     DosFreeMem (aspi_buf);
   aspi_buf = 0;
+
+  unlink (tmpAspi);		/* remove scsi descriptions */
+  strcpy (tmpAspi, "");
+
+  DBG (1, "OS/2: ASPI closed\n");
 }
 
 #endif /* USE_OS2_INTERFACE */
@@ -379,7 +574,7 @@ sanei_scsi_open (const char *dev, int *fdp,
 
   DBG_INIT ();
 
-#ifdef __linux__
+#if USE == LINUX_INTERFACE
   if (first_time)
     {
       char buf[32];
@@ -577,7 +772,7 @@ sanei_scsi_open (const char *dev, int *fdp,
     if (status.all != status_$ok)
       {
 	/* we have a failure, return an error code, and generate debug
-           output */
+	   output */
 	DBG (1, "sanei_scsi_open: acquire failed, Domain/OS status is %08x\n",
 	     status.all);
 	error_$print (status);
@@ -625,7 +820,8 @@ sanei_scsi_open (const char *dev, int *fdp,
   fd = open (dev, O_RDWR | O_EXCL);
   if (fd < 0)
     {
-      DBG (1, "sanei_scsi_open: open() failed: %s\n", strerror (errno));
+      DBG (1, "sanei_scsi_open: open of `%s' failed: %s\n",
+	   dev, strerror (errno));
       return SANE_STATUS_INVAL;
     }
 
@@ -761,7 +957,7 @@ sanei_scsi_close (int fd)
 
 
 #if USE == DOMAINOS_INTERFACE
-#define WE_HAVE_ASYNC_SCSI
+# define WE_HAVE_ASYNC_SCSI
 
 void
 sanei_scsi_req_flush_all (void)
@@ -780,6 +976,7 @@ SANE_Status
 sanei_scsi_req_enter (int fd, const void *src, size_t src_size,
 		      void *dst, size_t * dst_size, void **idp)
 {
+  SANEI_SCSI_Sense_Handler handler;
   static int index;
   static SANE_Status sane_status;
   static status_$t status;
@@ -877,21 +1074,26 @@ sanei_scsi_req_enter (int fd, const void *src, size_t src_size,
 	{
 	case scsi_check_condition:
 	  /* Call the sense handler, if defined */
-	  if (fd_info[com->fd].sense_handler)
-	    fd_info[com->fd].sense_handler (fd, ((u_char *) (fd_info[fd].pdata)) + DomainMaxDataSize, fd_info[com->fd].sense_handler_arg);
+	  handler = fd_info[com->fd].sense_handler;
+	  if (handler)
+	    (*handler) (fd, ((u_char *) fd_info[fd].pdata
+			     + DomainMaxDataSize),
+			fd_info[com->fd].sense_handler_arg);
 	  sane_status = SANE_STATUS_IO_ERROR;
 	  break;
 	case scsi_busy:
 	  sane_status = SANE_STATUS_DEVICE_BUSY;
 	  break;
 	default:
-	  DBG (0, "Error - Unrecognized SCSI status %08x returned from Enter command\n", SCSIStatus);
+	  DBG (0, "Error - Unrecognized SCSI status %08x returned from "
+	       "Enter command\n", SCSIStatus);
 	  sane_status = SANE_STATUS_IO_ERROR;
 	  exit (EXIT_FAILURE);
 	}
       break;
     default:
-      DBG (0, "Unmapped status (%08x) returned from Domain SANE Server\n", status.all);
+      DBG (0, "Unmapped status (%08x) returned from Domain SANE Server\n",
+	   status.all);
       sane_status = SANE_STATUS_IO_ERROR;
     }
 
@@ -932,10 +1134,13 @@ sanei_scsi_cmd (int fd, const void *src, size_t src_size,
 
 #if USE == LINUX_INTERFACE
 
+#include <ctype.h>
 #include <signal.h>
+
 #include <sys/time.h>
 
 #define WE_HAVE_ASYNC_SCSI
+#define WE_HAVE_FIND_DEVICES
 
 static int pack_id = 0;
 static int need_init = 1;
@@ -1120,25 +1325,25 @@ sanei_scsi_req_wait (void *id)
 	  if ((req->cdb.hdr.result != 0) ||
 	      ((req->cdb.hdr.sense_buffer[0] & 0x7f) != 0))
 	    {
- 	      SANEI_SCSI_Sense_Handler handler
+	      SANEI_SCSI_Sense_Handler handler
 		= fd_info[req->fd].sense_handler;
- 	      void *arg = fd_info[req->fd].sense_handler_arg;
- 
-	      DBG(1, "sanei_scsi_req_wait: SCSI command complained: %s\n",
- 		  strerror (req->cdb.hdr.result));
- 
- 	      if (req->cdb.hdr.result == EBUSY)
- 		status = SANE_STATUS_DEVICE_BUSY;
- 	      else if (handler)
+	      void *arg = fd_info[req->fd].sense_handler_arg;
+
+	      DBG (1, "sanei_scsi_req_wait: SCSI command complained: %s\n",
+		   strerror (req->cdb.hdr.result));
+
+	      if (req->cdb.hdr.result == EBUSY)
+		status = SANE_STATUS_DEVICE_BUSY;
+	      else if (handler)
 		/* sense handler should return SANE_STATUS_GOOD if it
 		   decided all was ok afterall */
- 		status = (*handler) (req->fd, req->cdb.hdr.sense_buffer, arg);
- 	      else
- 		status = SANE_STATUS_IO_ERROR;
- 	    }
+		status = (*handler) (req->fd, req->cdb.hdr.sense_buffer, arg);
+	      else
+		status = SANE_STATUS_IO_ERROR;
+	    }
 
 	  /* if we are ok so far, copy over the return data */
-	  if (status == SANE_STATUS_GOOD) 
+	  if (status == SANE_STATUS_GOOD)
 	    {
 	      if (req->dst)
 		memcpy (req->dst, req->cdb.data, nread);
@@ -1171,6 +1376,160 @@ sanei_scsi_cmd (int fd, const void *src, size_t src_size,
   return sanei_scsi_req_wait (id);
 }
 
+#define PROCFILE	"/proc/scsi/scsi"
+
+#define CSTRLEN(string_const)	(sizeof (string_const) - 1)
+
+static int
+get_devicename (int devnum, char *name, size_t name_len)
+{
+  int dev_fd, i;
+  static const struct
+    {
+      const char *prefix;
+      char base;
+    }
+  device_name_list[] =
+    {
+      {"/dev/sg",  'a'},
+      {"/dev/sg",  '0'},
+      {"/dev/uk",  '0'},
+      {"/dev/gsc", '0'},
+    };
+
+  for (i = 0; i < NELEMS (device_name_list); ++i)
+    {
+      snprintf (name, name_len, "%s%c", device_name_list[i].prefix,
+		device_name_list[i].base + devnum);
+
+      dev_fd = open (name, O_RDWR);
+      if (dev_fd >= 0)
+	{
+	  close (dev_fd);
+	  DBG (1, "searched device is %s\n", name);
+	  return 0;
+	}
+    }
+  return -1;
+}
+
+void
+sanei_scsi_find_devices (const char *findvendor, const char *findmodel,
+			 const char *findtype,
+			 int findbus, int findchannel, int findid, int findlun,
+			 SANE_Status (*attach) (const char *dev))
+{
+  size_t findvendor_len = 0, findmodel_len = 0, findtype_len = 0;
+  char vendor[32], model[32], type[32], revision[32];
+  int bus, channel, id, lun, number, i;
+  char line[256], dev_name[128];
+  const char *string;
+  FILE *proc_fp;
+  char *end;
+  const struct
+    {
+      const char *name;
+      size_t name_len;
+      int is_int;		/* integer valued? (not a string) */
+      union
+	{
+	  void *v;		/* avoids compiler warnings... */
+	  char *str;
+	  int *i;
+	}
+      u;
+    }
+  param[] =
+    {
+      {"Vendor:",  7, 0, { vendor }},
+      {"Model:",   6, 0, { model }},
+      {"Type:",    5, 0, { type }},
+      {"Rev:",     4, 0, { revision }},
+      {"scsi",     4, 1, { &bus }},
+      {"Channel:", 8, 1, { &channel }},
+      {"Id:",      3, 1, { &id }},
+      {"Lun:",     4, 1, { &lun }}
+    };
+
+  DBG_INIT ();
+
+  proc_fp = fopen (PROCFILE, "r");
+  if (!proc_fp)
+    {
+      DBG (1, "could not open %s for reading\n", PROCFILE);
+      return;
+    }
+
+  number = bus = channel = id = lun = -1;
+
+  vendor[0] = model[0] = type[0] = '\0';
+  if (findvendor)
+    findvendor_len = strlen (findvendor);
+  if (findmodel)
+    findmodel_len = strlen (findmodel);
+  if (findtype)
+    findtype_len = strlen (findtype);
+
+  while (!feof (proc_fp))
+    {
+      fgets (line, sizeof (line), proc_fp);
+      string = sanei_config_skip_whitespace (line);
+
+      while (*string)
+	{
+	  for (i = 0; i < NELEMS (param); ++i)
+	    {
+	      if (strncmp (string, param[i].name, param[i].name_len) == 0)
+		{
+		  string += param[i].name_len;
+		  string = sanei_config_skip_whitespace (string);
+		  if (param[i].is_int)
+		    {
+		      *param[i].u.i = strtol (string, &end, 10);
+		      string = (char *) end;
+		    }
+		  else
+		    {
+		      strncpy (param[i].u.str, string, 32);
+		      param[i].u.str[31] = '\0';
+		      while (*string && !isspace (*string))
+			++string;
+		    }
+		  string = sanei_config_skip_whitespace (string);
+
+		  if (param[i].u.v == &bus)
+		    ++number;
+		  break;
+		}
+	    }
+	  if (i >= NELEMS (param))
+	    ++string;		/* no match */
+	}
+
+      if ((findvendor && !vendor[0]) || (findmodel && !model[0])
+	  || (findtype && !type[0])
+	  || (findbus >= 0 && bus == -1) || (findchannel >= 0 && channel == -1)
+	  || (findlun >= 0 && lun == -1))
+	/* some info is still missing */
+	continue;
+
+      if ((!findvendor || strncmp (vendor, findvendor, findvendor_len) == 0)
+	  && (!findmodel || strncmp (model, findmodel, findmodel_len) == 0)
+	  && (!findtype || strncmp (type, findtype, findtype_len) == 0)
+	  && (findbus == -1 || bus == findbus)
+	  && (findchannel == -1 || channel == findchannel)
+	  && (findid == -1 || id == findid)
+	  && (findlun == -1 || lun == findlun)
+	  && get_devicename (number, dev_name, sizeof (dev_name)) >= 0
+	  && (*attach) (dev_name) != SANE_STATUS_GOOD)
+	return;
+
+      vendor[0] = model[0] = type[0] = 0;
+      bus = channel = id = lun = -1;
+    }
+  fclose (proc_fp);
+}
+
 #endif /* USE == LINUX_INTERFACE */
 
 
@@ -1479,6 +1838,138 @@ sanei_scsi_cmd (int fd, const void *src, size_t src_size,
 #endif /* USE == SCO_OS5_INTERFACE */
 
 #if USE == OS2_INTERFACE
+
+#define WE_HAVE_FIND_DEVICES
+
+static int
+get_devicename (int bus, int target, int lun, char *name, size_t name_len)
+{
+  snprintf (name, name_len, "b%dt%dl%d", bus, target, lun);
+  DBG (1, "OS/2 searched device is %s\n", name);
+  return 0;
+}
+
+void
+sanei_scsi_find_devices (const char *findvendor, const char *findmodel,
+			 const char *findtype,
+		      int findbus, int findchannel, int findid, int findlun,
+			 SANE_Status (*attach) (const char *dev))
+{
+  size_t findvendor_len = 0, findmodel_len = 0, findtype_len = 0;
+  char vendor[32], model[32], type[32], revision[32];
+  int bus, channel, id, lun, number, i;
+  char line[256], dev_name[128];
+  const char *string;
+  FILE *proc_fp;
+  char *end;
+  const struct
+    {
+      const char *name;
+      size_t name_len;
+      int is_int;		/* integer valued? (not a string) */
+      union
+	{
+	  void *v;		/* avoids compiler warnings... */
+	  char *str;
+	  int *i;
+	}
+      u;
+    }
+  param[] =
+    {
+      {"Vendor:",  7, 0, { vendor }},
+      {"Model:",   6, 0, { model }},
+      {"Type:",    5, 0, { type }},
+      {"Rev:",     4, 0, { revision }},
+      {"scsi",     4, 1, { &bus }},
+      {"Channel:", 8, 1, { &channel }},
+      {"Id:",      3, 1, { &id }},
+      {"Lun:",     4, 1, { &lun }}
+  };
+
+  DBG_INIT ();
+
+  open_aspi ();			/* open aspi manager if not already done */
+
+  proc_fp = fopen (tmpAspi, "r");
+  if (!proc_fp)
+    {
+      DBG (1, "could not open %s for reading\n", tmpAspi);
+      return;
+    }
+
+  number = bus = channel = id = lun = -1;
+
+  vendor[0] = model[0] = type[0] = '\0';
+  if (findvendor)
+    findvendor_len = strlen (findvendor);
+  if (findmodel)
+    findmodel_len = strlen (findmodel);
+  if (findtype)
+    findtype_len = strlen (findtype);
+
+  while (!feof (proc_fp))
+    {
+      if (!fgets (line, sizeof (line), proc_fp))
+	break;			/* at eof exit */
+
+      string = sanei_config_skip_whitespace (line);
+
+      while (*string)
+	{
+	  for (i = 0; i < NELEMS (param); ++i)
+	    {
+	      if (strncmp (string, param[i].name, param[i].name_len) == 0)
+		{
+		  string += param[i].name_len;
+		  string = sanei_config_skip_whitespace (string);
+		  if (param[i].is_int)
+		    {
+		      *param[i].u.i = strtol (string, &end, 10);
+		      string = (char *) end;
+		    }
+		  else
+		    {
+		      strncpy (param[i].u.str, string, 32);
+		      param[i].u.str[31] = '\0';
+		      while (*string && !isspace (*string))
+			++string;
+		    }
+		  string = sanei_config_skip_whitespace (string);
+
+		  if (param[i].u.v == &bus)
+		    ++number;
+		  break;
+		}
+	    }
+	  if (i >= NELEMS (param))
+	    ++string;		/* no match */
+	}
+
+      if ((findvendor && !vendor[0]) || (findmodel && !model[0])
+	  || (findtype && !type[0])
+       || (findbus >= 0 && bus == -1) || (findchannel >= 0 && channel == -1)
+	  || (findlun >= 0 && lun == -1))
+	/* some info is still missing */
+	continue;
+
+      if ((!findvendor || strncmp (vendor, findvendor, findvendor_len) == 0)
+	  && (!findmodel || strncmp (model, findmodel, findmodel_len) == 0)
+	  && (!findtype || strncmp (type, findtype, findtype_len) == 0)
+	  && (findbus == -1 || bus == findbus)
+	  && (findchannel == -1 || channel == findchannel)
+	  && (findid == -1 || id == findid)
+	  && (findlun == -1 || lun == findlun)
+	  && get_devicename (bus, id, lun, dev_name, sizeof (dev_name)) >= 0
+	  && (*attach) (dev_name) != SANE_STATUS_GOOD)
+	return;
+
+      vendor[0] = model[0] = type[0] = 0;
+      bus = channel = id = lun = -1;
+    }
+  fclose (proc_fp);
+}
+
 /* XXX untested code! */
 SANE_Status
 sanei_scsi_cmd (int fd, const void *src, size_t src_size,
@@ -1702,7 +2193,7 @@ sanei_scsi_cmd (int fd, const void *src, size_t src_size,
 #if USE == SOLARIS_SG_INTERFACE
 
 #ifndef CCS_SENSE_LEN
-#define CCS_SENSE_LEN 18
+# define CCS_SENSE_LEN 18
 #endif
 
 SANE_Status
@@ -1911,3 +2402,16 @@ sanei_scsi_req_flush_all (void)
 }
 
 #endif /* WE_HAVE_ASYNC_SCSI */
+
+#ifndef WE_HAVE_FIND_DEVICES
+
+void
+sanei_scsi_find_devices (const char *findvendor, const char *findmodel,
+			 const char *findtype,
+			 int findbus, int findchannel, int findid, int findlun,
+			 SANE_Status (*attach) (const char *dev))
+{
+  DBG (1, "sanei_scsi_find_devices: not implemented for this platform\n");
+}
+
+#endif /* WE_HAVE_FIND_DEVICES */
